% Non-linear turbine model without eta_i implementation - low head turbine

clear all
close all
clc

% Design parameters from excel sheet - choose turbine by specifying column
 design=table2array(readtable('Hillchart_r3_medBEP.xlsx','Sheet','Turbin_design','Range','R9:R80'));

% necessary model input:
 alfa_1R=design(49,1); sigma=design(53,1); psi=design(54,1); xi=design(56,1);

%--------------------------------------------------------------------------
% common for plot 1 - 4:
%--------------------------------------------------------------------------
omega_query=linspace(0,1.9,191);                    %omega=1 included
y_query=linspace(0.3,1.3,11);                       %y=1 included
h=1;


%--------------------------------------------------------------------------
% 1. Flow - speed plot at different opening degrees and const rated head
%--------------------------------------------------------------------------
figure(1)
axis([0 2 0 1.6])
xlabel('$\tilde{\omega}$ [-]','Interpreter','latex','FontSize',16)
ylabel('q [-]','Interpreter','latex','FontSize',16)
title('Model: Flow - speed at h=1','Interpreter','latex','FontSize',16)
hold on

q=zeros(length(y_query),length(omega_query));       %each row const opening, each column const speed
for i=1:length(y_query)
    q(i,:)=y_query(i)*sqrt(h-sigma*(omega_query.^2-1));
    
    %plot one curve per opening degree
    txt=['y = ',num2str(y_query(i))];
    plot(omega_query,q(i,:),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10,'Location','southeast')
end

hold off
legend show

% find dq/domega at q=omega=y=h=1
pos_omega=find(omega_query==1);
pos_y=find(y_query==1);
q_test=q(pos_y,pos_omega)                           %should be 1

% using a central difference scheme
dqdomega=(q(pos_y,pos_omega+1)-q(pos_y,pos_omega-1))/(omega_query(pos_omega+1)-omega_query(pos_omega-1));

% using inbuildt gradient function
f=gradient(q(pos_y,:));
dqdomega_check=f(pos_omega)/(0.5*(omega_query(pos_omega+1)-omega_query(pos_omega-1)));


%--------------------------------------------------------------------------
% 2. Torque - speed plot at different opening degrees and const rated head
%--------------------------------------------------------------------------
figure(2)
axis([0 2 0 1.6])
xlabel('$\tilde{\omega}$ [-]','Interpreter','latex','FontSize',16)
ylabel('t [-]','Interpreter','latex','FontSize',16)
title('Model: Torque - speed at h=1','Interpreter','latex','FontSize',16)
hold on

t=zeros(length(y_query),length(omega_query));       %each row const opening, each column const speed
for i=1:length(y_query)
    alfa_1=asin(y_query(i)*sin(alfa_1R*pi/180));    %radians
    m_S=xi*q(i,:)./y_query(i)*(cos(alfa_1)+tan(alfa_1R*pi/180)*sin(alfa_1)); 
    t(i,:)=q(i,:).*(m_S - psi.*omega_query);
    
    %plot one curve per opening degree
    txt=['y = ',num2str(y_query(i))];
    plot(omega_query,t(i,:),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10,'Location','northeast')
end

hold off
legend show

% find dt/domega at t=omega=y=h=1
pos_omega=find(omega_query==1);
pos_y=find(y_query==1);
t_test=t(pos_y,pos_omega)                           %should be 1

% using a central difference scheme
dtdomega=(t(pos_y,pos_omega+1)-t(pos_y,pos_omega-1))/(omega_query(pos_omega+1)-omega_query(pos_omega-1));

% using inbuildt gradient function
f=gradient(t(pos_y,:));
dtdomega_check=f(pos_omega)/(0.5*(omega_query(pos_omega+1)-omega_query(pos_omega-1)));


%------------------------------------------------------------------------------
% 3. Mech. power - speed plot at different opening degrees and const rated head
%------------------------------------------------------------------------------
figure(3)
axis([0 2 0 1.6])
xlabel('$\tilde{\omega}$ [-]','Interpreter','latex','FontSize',16)
ylabel('p = t*$\tilde{\omega}$ [-]','Interpreter','latex','FontSize',16)
title('Model: Output power - speed at h=1','Interpreter','latex','FontSize',16)
hold on

p=zeros(length(y_query),length(omega_query));       %each row const opening, each column const speed
for i=1:length(y_query)
    p(i,:)=t(i,:).*omega_query;
    
    %plot one curve per opening degree
    txt=['y = ',num2str(y_query(i))];
    plot(omega_query,p(i,:),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10,'Location','northeast')
end

hold off
legend show


%-----------------------------------------------------------------------------
% 4. Efficiency - speed plot at different opening degrees and const rated head
%-----------------------------------------------------------------------------
figure(4)
axis([0 2 0 1.1])
xlabel('$\tilde{\omega}$ [-]','Interpreter','latex','FontSize',16)
ylabel('$\tilde{\eta}$ [-]','Interpreter','latex','FontSize',16)
title('Model: Efficiency - speed at h=1','Interpreter','latex','FontSize',16)
hold on

eta=zeros(length(y_query),length(omega_query));     %each row const opening, each column const speed
for i=1:length(y_query)
    
    eta(i,:)=(t(i,:).*omega_query)./(q(i,:).*1);    %since h=1
    
    %plot one curve per opening degree
    txt=['y = ',num2str(y_query(i))];
    plot(omega_query,eta(i,:),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10,'Location','south')
end

hold off
legend show



%--------------------------------------------------------------------------
% common for plot 5 - 8:
%--------------------------------------------------------------------------
y_query=linspace(0,1.5,151);                        %y=1 included
omega_query=linspace(0.95,1.4,10);                  %omega=1 included
h=1;


%--------------------------------------------------------------------------
% 5. Flow - opening degree plot at different speeds and const rated head
%--------------------------------------------------------------------------
figure(5)
axis([0 2 0 1.5])
xlabel('y [-]','Interpreter','latex','FontSize',16)
ylabel('q [-]','Interpreter','latex','FontSize',16)
title('Model: Flow - opening degree at h=1','Interpreter','latex','FontSize',16)
hold on

q=zeros(length(y_query),length(omega_query));       %each row const opening, each column const speed
for j=1:length(omega_query)
    q(:,j)=y_query.*sqrt(h-sigma*(omega_query(j)^2-1));
    
    %plot one curve per speed
    txt=['$\tilde{\omega} = $',num2str(omega_query(j))];
    plot(y_query,q(:,j),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show

% find dq/dy at q=y=omega=h=1
pos_y=find(y_query==1);
pos_omega=find(omega_query==1);
q_test=q(pos_y,pos_omega)                           %should be 1

% using a central difference scheme
dqdy=(q(pos_y+1,pos_omega)-q(pos_y-1,pos_omega))/(y_query(pos_y+1)-y_query(pos_y-1));

% using inbuildt gradient function
f=gradient(q(:,pos_omega));
dqdy_check=f(pos_y)/(0.5*(y_query(pos_y+1)-y_query(pos_y-1)));


%--------------------------------------------------------------------------
% 6. Torque - opening degree plot at different speeds and const rated head
%--------------------------------------------------------------------------
figure(6)
axis([0 2 0 1.6])
xlabel('y [-]','Interpreter','latex','FontSize',16)
ylabel('t [-]','Interpreter','latex','FontSize',16)
title('Model: Torque - opening degree at h=1','Interpreter','latex','FontSize',16)
hold on

t=zeros(length(y_query),length(omega_query));       %each row const opening, each column const speed
alfa_1=asin(y_query.*sin(alfa_1R*pi/180));          %radians

for j=1:length(omega_query)
    m_S=xi*((q(:,j)')./y_query).*(cos(alfa_1)+tan(alfa_1R*pi/180)*sin(alfa_1));
    t(:,j)=(q(:,j)').*(m_S - psi.*omega_query(j));
    
    %plot one curve per speed
    txt=['$\tilde{\omega} = $',num2str(omega_query(j))];
    plot(y_query,t(:,j),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10,'Location','northwest')
end

hold off
legend show

% find dt/dy at t=y=omega=h=1
pos_omega=find(omega_query==1);
pos_y=find(y_query==1);
t_test=t(pos_y,pos_omega)                           %should be 1

% using a central difference scheme
dtdy=(t(pos_y+1,pos_omega)-t(pos_y-1,pos_omega))/(y_query(pos_y+1)-y_query(pos_y-1));

% using inbuildt gradient function
f=gradient(t(:,pos_omega));
dtdy_check=f(pos_y)/(0.5*(y_query(pos_y+1)-y_query(pos_y-1)));


%------------------------------------------------------------------------------
% 7. Mech. power - opening degree plot at different speeds and const rated head
%------------------------------------------------------------------------------
figure(7)
axis([0 2 0 1.5])
xlabel('y [-]','Interpreter','latex','FontSize',16)
ylabel('p = t*$\tilde{\omega}$ [-]','Interpreter','latex','FontSize',16)
title('Model: Output power - opening degree at h=1','Interpreter','latex','FontSize',16)
hold on

p=zeros(length(y_query),length(omega_query));       %each row const opening, each column const speed

for j=1:length(omega_query)
    p(:,j)=t(:,j).*omega_query(j);
    
    %plot one curve per speed
    txt=['$\tilde{\omega} = $',num2str(omega_query(j))];
    plot(y_query,p(:,j),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show


%-----------------------------------------------------------------------------
% 8. Efficiency - opening degree plot at different speeds and const rated head
%-----------------------------------------------------------------------------
figure(8)
axis([0 2 0 1.1])
xlabel('y [-]','Interpreter','latex','FontSize',16)
ylabel('$\tilde{\eta}$ [-]','Interpreter','latex','FontSize',16)
title('Model: Efficiency - opening degree at h=1','Interpreter','latex','FontSize',16)
hold on

eta=zeros(length(y_query),length(omega_query));     %each row const opening, each column const speed
for j=1:length(omega_query)
    
    eta(:,j)=(t(:,j).*omega_query(j))./(q(:,j).*1); %since h=1
    
    %plot one curve per speed
    txt=['$\tilde{\omega} = $',num2str(omega_query(j))];
    plot(y_query,eta(:,j),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show



%--------------------------------------------------------------------------
% common for plot 9 - 11:
%--------------------------------------------------------------------------
q_query=linspace(0,1.6,81);                         %q=1 included
omega_query=linspace(0.95,1.4,10);                  %omega=1 included
h=1;


%--------------------------------------------------------------------------
% 9. Torque - flow plot at different speeds and const rated head
%--------------------------------------------------------------------------
figure(9)
axis([0 2 0 1.5])
xlabel('q [-]','Interpreter','latex','FontSize',16)
ylabel('t [-]','Interpreter','latex','FontSize',16)
title('Model: Torque - flow at h=1','Interpreter','latex','FontSize',16)
hold on

t=zeros(length(q_query),length(omega_query));       %each row const flow, each column const speed
for j=1:length(omega_query)
    
    qdivy=sqrt(h-sigma*(omega_query(j)^2-1));
    y=q_query./qdivy;
    alfa_1=asin(y.*sin(alfa_1R*pi/180));            %radians
    m_S=xi*qdivy*(cos(alfa_1)+tan(alfa_1R*pi/180)*sin(alfa_1));
    
    t(:,j)=q_query.*(m_S - psi.*omega_query(j));
    
    %plot one curve per speed
    txt=['$\tilde{\omega} = $',num2str(omega_query(j))];
    plot(q_query,t(:,j),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show


%--------------------------------------------------------------------------
% 10. Mech. power - flow plot at different speeds and const rated head
%--------------------------------------------------------------------------
figure(10)
axis([0 2 0 1.5])
xlabel('q [-]','Interpreter','latex','FontSize',16)
ylabel('p = t*$\tilde{\omega}$ [-]','Interpreter','latex','FontSize',16)
title('Model: Output power - flow at h=1','Interpreter','latex','FontSize',16)
hold on

p=zeros(length(q_query),length(omega_query));       %each row const flow, each column const speed
for j=1:length(omega_query)
    
    p(:,j)=t(:,j).*omega_query(j);
    
    %plot one curve per speed
    txt=['$\tilde{\omega} = $',num2str(omega_query(j))];
    plot(q_query,p(:,j),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show


%--------------------------------------------------------------------------
% 11. Efficiency - flow plot at different speeds and const rated head
%--------------------------------------------------------------------------
figure(11)
axis([0 2 0 1.1])
xlabel('q [-]','Interpreter','latex','FontSize',16)
ylabel('$\tilde{\eta}$ [-]','Interpreter','latex','FontSize',16)
title('Model: Efficiency - flow at h=1','Interpreter','latex','FontSize',16)
hold on

eta=zeros(length(q_query),length(omega_query));     %each row const flow, each column const speed
for j=2:length(omega_query)
    
    eta(:,j)=(t(:,j).*omega_query(j))./(q_query'.*1);%since h=1
    
    %plot one curve per speed
    txt=['$\tilde{\omega} = $',num2str(omega_query(j))];
    plot(q_query,eta(:,j),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10,'Location','south')
end

hold off
legend show



%--------------------------------------------------------------------------
% common for plot 12 - 14:
%--------------------------------------------------------------------------
h_query=linspace(0,1.5,151);                        %h=1 included
y_query=linspace(0.3,1.3,11);                       %y=1 included
omega=1;


%--------------------------------------------------------------------------
% 12. Flow - head plot at different opening degrees and const rated speed
%--------------------------------------------------------------------------
figure(12)
axis([0 2 0 1.8])
xlabel('h [-]','Interpreter','latex','FontSize',16)
ylabel('q [-]','Interpreter','latex','FontSize',16)
title('Model: Flow - head at $\tilde{\omega} = 1$','Interpreter','latex','FontSize',16)
hold on

q=zeros(length(y_query),length(h_query));           %each row const opening, each column const head
for i=1:length(y_query)
    q(i,:)=y_query(i)*sqrt(h_query-sigma*(omega^2-1));
    
    %plot one curve per opening degree
    txt=['y = ',num2str(y_query(i))];
    plot(h_query,q(i,:),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show

% find dq/dh at q=h=y=omega=1 
pos_h=find(h_query==1);
pos_y=find(y_query==1);
q_test=q(pos_y,pos_h)                               %should be 1

% using a central difference scheme
dqdh=(q(pos_y,pos_h+1)-q(pos_y,pos_h-1))/(h_query(pos_h+1)-h_query(pos_h-1));

% using inbuildt gradient function
f=gradient(q(pos_y,:));
dqdh_check=f(pos_h)/(0.5*(h_query(pos_h+1)-h_query(pos_h-1)));


%--------------------------------------------------------------------------
% 13. Torque - head plot at different opening degrees and const rated speed
%--------------------------------------------------------------------------
figure(13)
axis([0 2 0 1.5])
xlabel('h [-]','Interpreter','latex','FontSize',16)
ylabel('t [-]','Interpreter','latex','FontSize',16)
title('Model: Torque - head at $\tilde{\omega} = 1$','Interpreter','latex','FontSize',16)
hold on

t=zeros(length(y_query),length(h_query));           %each row const opening, each column const head
for i=1:length(y_query)
    
    qdivy=sqrt(h_query-sigma*(omega^2-1));
    alfa_1=asin(y_query(i)*sin(alfa_1R*pi/180));    %radians
    m_S=xi*qdivy.*(cos(alfa_1)+tan(alfa_1R*pi/180)*sin(alfa_1));
    
    t(i,:)=q(i,:).*(m_S - psi*omega);
    
    %plot one curve per opening degree
    txt=['y = ',num2str(y_query(i))];
    plot(h_query,t(i,:),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show

% find dt/dh at q=h=y=omega=1 
pos_h=find(h_query==1);
pos_y=find(y_query==1);
t_test=t(pos_y,pos_h)                               %should be 1

% using a central difference scheme
dtdh=(t(pos_y,pos_h+1)-t(pos_y,pos_h-1))/(h_query(pos_h+1)-h_query(pos_h-1));

% using inbuildt gradient function
f=gradient(t(pos_y,:));
dtdh_check=f(pos_h)/(0.5*(h_query(pos_h+1)-h_query(pos_h-1)));


%------------------------------------------------------------------------------
% 14. Efficiency - head plot at different opening degrees and const rated speed
%------------------------------------------------------------------------------
figure(14)
axis([0 2 0 1.1])
xlabel('h [-]','Interpreter','latex','FontSize',16)
ylabel('$\tilde{\eta}$ [-]','Interpreter','latex','FontSize',16)
title('Model: Efficiency - head at $\tilde{\omega} = 1$','Interpreter','latex','FontSize',16)
hold on

eta=zeros(length(y_query),length(h_query));         %each row const opening, each column const head
for i=1:length(y_query)
    
    eta(i,:)=(t(i,:).*1)./(q(i,:).*h_query);        %since omega=1
    
    %plot one curve per opening degree
    txt=['y = ',num2str(y_query(i))];
    plot(h_query,eta(i,:),'-','LineWidth',1,'DisplayName',txt);
    set(legend,'Interpreter','latex','FontSize',10)
end

hold off
legend show



%--------------------------------------------------------------------------
% Finding the characteristics at BEP
%--------------------------------------------------------------------------

a_11 = dqdh;
a_12 = dqdy;
a_13 = dqdomega;
a_21 = dtdh/a_11;
a_22 = dtdy - a_21*a_12;
a_23 = dtdomega - a_21*a_13;

check = a_21 + a_22                             %should be =1

